#######################################################
# 画像の形状とサイズを調整(カラー画像)
#
# Shin.Onda 2023/6//3 all rights reserved.
#
#######################################################
import re
import numpy as np
import random
import cv2

class AdjustImage :

    ######################################
    # キャンバスの生成
    # パラメータ
    #   height= 高さ
    #   width = 幅
    #   imgBack = 背景('R':乱数)
    # 戻り値 イメージリスト
    ######################################
    def createCanvas(self, height, width, imgBack) :
        if imgBack == 'R' : # 乱数
            imgNew = [[[int(random.random()*255) for i in range(3)]  for i in range(width) ] for i in range(height)]
        elif imgBack == 'B' : # 黒
            imgNew = [[[0] * 3] * width] * height
        else : # 白
          imgNew = [[[255] * 3] * width] * height
        return np.array(imgNew).astype(np.uint8)

    ######################################
    # 正方形に切り出し(短辺にあわせる)
    #   左寄り、中央、右寄りのイメージのリストを返す
    # パラメータ
    #   paraImg = イメージ
    # 戻り値 イメージリスト
    ######################################
    def cutSquare(self, paramImg) :
        # 画像パラメータ
        img = paramImg.copy()
        orgH, orgW, org_c = img.shape
        
        imgList = []
        # 短辺に合わせる
        if orgH > orgW :
            # 縦長
            basesize = orgW
            syCenter = int((orgH - basesize)/2)
            syBottom = orgH - basesize
            imgList.append(img[0       :basesize,          0:basesize])
            imgList.append(img[syCenter:syCenter+basesize, 0:basesize])
            imgList.append(img[syBottom:syBottom+basesize, 0:basesize])
        else :
            # 横長
            basesize = orgH
            sxCenter = int((orgW - basesize)/2)
            sxRight  = orgW - basesize
            imgList.append(img[0:basesize, 0       :basesize         ])
            imgList.append(img[0:basesize, sxCenter:sxCenter+basesize])
            imgList.append(img[0:basesize, sxRight :sxRight+basesize ])

        return imgList

    ######################################
    # 正方形(長辺にあわせる)
    # パラメータ
    #   paraImg = イメージ
    #   imgBack = 背景('R':乱数)
    # 戻り値 イメージ
    ######################################
    def adjustSquare(self, paramImg, imgBack='R') :
        # 画像パラメータ
        img = paramImg.copy()
        orgH, orgW, org_c = img.shape
        
        if orgH > orgW :
            # 縦長
            basesize = orgH
            canvas = self.createCanvas(basesize, basesize, imgBack)
            sx = int((basesize-orgW)/2)
            canvas[0:basesize, sx:sx+orgW] = img
        else :
            # 横長
            basesize = orgW
            canvas = self.createCanvas(basesize, basesize, imgBack)
            sy = int((basesize-orgH)/2)
            canvas[sy:sy+orgH, 0:basesize] = img

        return canvas


    ######################################
    # 正方形を拡大縮小
    # パラメータ
    #   paraImg = イメージ
    #   imgBack = 背景('R':乱数)
    #   magSmall = 拡大縮小　最小％
    #   magLarge = 拡大縮小　最大％
    #   magStep  = 間隔
    # 戻り値 イメージリスト
    ######################################
    def zoomSquare(self, paramImg, imgBack='R', magSmall=50, magLarge=150, magStep=10) :
        # 画像パラメータ
        orgH, orgW, orgC = paramImg.shape

        # 縮小拡大のループ
        mag = 1.0
        imgList = []
        for i in range(0, magStep) :
            canvas = paramImg.copy()

            # 倍率に合わせたサイズ変更
            if magStep > 1 :
                delta = (magLarge - magSmall) / (magStep-1)
                mag   = (magSmall + (delta * i)) / 100.0
            else :
                mag   = magSmall / 100.0
            print('Zoom {0:06.4f}'.format(mag))

            newImg = cv2.resize(canvas, dsize=(int(mag * orgW), int(mag * orgH)))
            newH, newW, newC = newImg.shape

            if mag > 1 :
                # 拡大
                sx = int((newW - orgW) / 2)
                ex = sx + orgW
                sy = int((newH - orgH) / 2)
                ey = sy + orgH
                newCanvas = newImg[sy:ey, sx:ex]
            else :
                # 縮小
                sx = int((orgW - newW) / 2)
                ex = sx + newW
                sy = int((orgH - newH) / 2)
                ey = sy + newH
                newCanvas = self.createCanvas(orgH, orgW, imgBack)
                newCanvas[sy:ey, sx:ex] = newImg
            # 戻り値リストに追加
            imgList.append(newCanvas)
        
        return imgList

