##################################################
# 動画から画像を切り出す
#   マウス制御クラス
#
# Shin.Onda 2023/6//3 all rights reserved.
##################################################
import cv2
import numpy as np
import tkinter as tk
import tkinter.messagebox as messagebox
import tkinter.simpledialog as simpledialog

class MouseCtrl :
    ###################
    # メンバ変数
    ###################
    moveFlag     = False
    writeFlag    = False
    processFlag  = False
    writeCounter = 0
    frameNo      = 0

    # 動作中フラグ
    def getMoveFlag(self) :
        return self.moveFlag
    def setMoveFlag(self, moveFlag) :
        self.moveFlag = moveFlag
    # 書き込みフラグ
    def getWriteFlag(self) :
        return self.writeFlag
    def setWriteFlag(self, writeFlag) :
        self.writeFlag = writeFlag
    # 書き込み処理中フラグ
    def getProcessFlag(self) :
        return self.processFlag
    def setProcessFlag(self, processFlag) :
        self.processFlag = processFlag
    # 書き込みカウンタ
    def getWriteCounter(self) :
        return self.writeCounter
    def setWriteCounter(self, writeCounter) :
        self.writeCounter = writeCounter
    # フレーム番号
    def getFrameNo(self) :
        return self.frameNo
    def setFrameNo(self, frameNo) :
        self.frameNo = frameNo

    ###############################
    # 操作用のボタン
    ###############################
    ###############################
    # コンポーネントの範囲内チェック
    #   x   = 座標位置X
    #   y   = 座標位置Y
    #   box = 範囲
    # 戻り値 範囲内=True, 範囲外=False
    ###############################
    def inBox(self, x, y, box) :
        if (x >= box[0] and x <= box[0]+box[2]) and (y >= box[1] and y <= box[1]+box[3]) :
            return True
        return False

    ###############################
    # 動画サイズに合わせたサイズを求める
    #   orgW = イメージ幅
    #   orgH = イメージ高さ
    # 戻り値 サイズ
    ###############################
    def resizeBtn(self, size, orgW, orgH) :
        std = 512
        if orgW > orgH :
            newSize = int(size * orgW / std)
        else :
            newSize = int(size * orgH / std)
        return newSize

    ###############################
    # 動画の再生開始・停止ボタンの表示
    #   img  = イメージ
    #   orgW = イメージ幅
    #   orgH = イメージ高さ
    ###############################
    def drawStartStop(self, img, orgW, orgH) :
        lineSize = self.resizeBtn(10, orgW, orgH)
        margin = self.resizeBtn(10, orgW, orgH)
        clr = (255, 255, 255)
        if self.getProcessFlag() :
            clr = (127, 127, 127)

        # 表示位置とサイズを獲得
        sx, sy, w, h = self.getBoxStartStop(orgW, orgH)
        sx += margin
        sy += margin
        w  -= margin*2
        h  -= margin*2
        
        if self.getMoveFlag() :
            # 動作中は停止ボタン
            cv2.rectangle(img, (sx,   sy), (sx,   sy+h), clr, thickness=lineSize, lineType=cv2.LINE_4)
            cv2.rectangle(img, (sx+w, sy), (sx+w, sy+h), clr, thickness=lineSize, lineType=cv2.LINE_4)
        else :
            # 停止中は開始ボタン
            pts = np.array([(sx, sy), (sx+w, sy+int(h/2)), (sx, sy+h)])
            cv2.polylines(img, [pts], isClosed=True, color=clr, thickness=lineSize, lineType=cv2.LINE_4)
    
    ###############################
    # 動画の開始・停止ボタンの表示位置とサイズ
    #   orgW = イメージ幅
    #   orgH = イメージ高さ
    # 戻り値 座標X, 座標Y, 幅, 高さ
    ###############################
    def getBoxStartStop(self, orgW, orgH) :
        w = self.resizeBtn(80, orgW, orgH)
        h = self.resizeBtn(80, orgW, orgH)
        sx = int(orgW/2 - w/2)
        sy = int(orgH/2 - h/2)
        return sx, sy, w, h
    
    ###############################
    # 画像切り出し開始・停止ボタンの表示
    #   img  = イメージ
    #   orgW = イメージ幅
    #   orgH = イメージ高さ
    ###############################
    def drawWriteButton(self, img, orgW, orgH) :
        lineSize = self.resizeBtn(10, orgW, orgH)
        clrOn  = (  0, 0, 255)  # 開始ボタン（書き込み停止中）の色
        clrOff = (255, 0,   0)  # 停止ボタン（書き込み中）の色
        if self.getProcessFlag() :
            clrOn  = (127, 127, 127)
            clrOff = (127, 127, 127)

        # ボタンの表示位置とサイズを獲得
        sx, sy, w, h = self.getBoxWriteButton(orgW, orgH)
        cx = int(sx + w/2)
        cy = int(sy + h/2)
        r  = int(w/2)-lineSize

        if self.getWriteFlag() :
            # 書き込み中は、終了ボタン
            cv2.circle(img, (cx, cy), r, clrOn,  thickness=lineSize, lineType=cv2.LINE_AA)
        else :
            # 書き込み開始ボタン
            cv2.circle(img, (cx, cy), r, clrOff, thickness=lineSize, lineType=cv2.LINE_AA)
    
    ###############################
    # 画像切り出し開始・停止ボタンの表示位置とサイズ
    #   orgW = イメージ幅
    #   orgH = イメージ高さ
    # 戻り値 座標X, 座標Y, 幅, 高さ
    ###############################
    def getBoxWriteButton(self, orgW, orgH) :
        w = self.resizeBtn(60, orgW, orgH)
        h = self.resizeBtn(60, orgW, orgH)
        sx = orgW - w
        sy = 0
        return sx, sy, w, h
    
    ###############################
    # 読み飛ばしボタンの表示
    #   img  = イメージ
    #   orgW = イメージ幅
    #   orgH = イメージ高さ
    ###############################
    def drawFastButton(self, img, orgW, orgH) :
        lineSize = self.resizeBtn(5, orgW, orgH)
        margin = self.resizeBtn(10, orgW, orgH)
        angleSize = self.resizeBtn(20, orgW, orgH)
        clr = (0, 0, 255)
        if self.getProcessFlag() :
            clr = (127, 127, 127)
    
        # ボタンの表示位置とサイズ
        sx, sy, w, h = self.getBoxFastButton(orgW, orgH)
        w -= margin
        h -= margin
    
        na = np.array([(sx+angleSize, sy), (sx+w, sy+int(h/2)), (sx+angleSize, sy+h)])
        cv2.polylines(img, [na], isClosed=False, color=clr, thickness=lineSize, lineType=cv2.LINE_4)
        na[:,0] -= angleSize
        cv2.polylines(img, [na], isClosed=False, color=clr, thickness=lineSize, lineType=cv2.LINE_4)
    
    ###############################
    # 読み飛ばしボタンの表示位置とサイズ
    #   orgW = イメージ幅
    #   orgH = イメージ高さ
    # 戻り値 座標X, 座標Y, 幅, 高さ
    ###############################
    def getBoxFastButton(self, orgW, orgH) :
        w = self.resizeBtn(50, orgW, orgH)
        h = self.resizeBtn(50, orgW, orgH)
        sx = int(orgW - w)
        sy = int(orgH - h)
        return sx, sy, w, h
    
    ###############################
    # 巻き戻しボタンの表示
    #   img  = イメージ
    #   orgW = イメージ幅
    #   orgH = イメージ高さ
    ###############################
    def drawBackButton(self, img, orgW, orgH) :
        lineSize =self.resizeBtn( 5, orgW, orgH)
        margin = self.resizeBtn(10, orgW, orgH)
        angleSize = self.resizeBtn(20, orgW, orgH)
        clr = (0, 0, 255)
        if self.getProcessFlag() :
            clr = (127, 127, 127)
     
        # ボタンの表示位置とサイズ
        sx, sy, w, h = self.getBoxBackButton(orgW, orgH)
        sx += margin
        w  -= margin
        h  -= margin
    
        na = np.array([(sx+w-angleSize, sy), (sx, sy+int(h/2)), (sx+w-angleSize, sy+h)])
        cv2.polylines(img, [na], isClosed=False, color=clr, thickness=lineSize, lineType=cv2.LINE_4)
        na[:,0] += angleSize
        cv2.polylines(img, [na], isClosed=False, color=clr, thickness=lineSize, lineType=cv2.LINE_4)
    
    ###############################
    # 巻き戻しボタンの表示位置とサイズ
    #   orgW = イメージ幅
    #   orgH = イメージ高さ
    # 戻り値 座標X, 座標Y, 幅, 高さ
    ###############################
    def getBoxBackButton(self, orgW, orgH) :
        w = self.resizeBtn(50, orgW, orgH)
        h = self.resizeBtn(50, orgW, orgH)
        sx = 0
        sy = int(orgH - h)
        return sx, sy, w, h
    
    ###############################
    # スライドバーの表示
    #   img      = イメージ
    #   progress = プログレス(現在位置)
    #   orgW     = イメージ幅
    #   orgH     = イメージ高さ
    ###############################
    def drawSlide(self, img, progress, orgW, orgH) :
        lineSize = self.resizeBtn(5, orgW, orgH)
        clrbar   = (127, 127, 127)
        clrpoint = (255, 255, 255)
    
        # 範囲獲得
        sx, sy, w, h = self.getBoxSlide(orgW, orgH)
    
        # スライドバー
        cv2.rectangle(img, (sx, sy), (sx+w, sy+h), clrbar, thickness=lineSize, lineType=cv2.LINE_4)
        # 現在位置
        p = int(progress * w)
        cv2.rectangle(img, (sx+p, sy), (sx+p, sy+h), clrpoint, thickness=lineSize, lineType=cv2.LINE_4)
        
    ###############################
    # スライドバー表示位置とサイズ
    #   orgW = イメージ幅
    #   orgH = イメージ高さ
    # 戻り値 座標X, 座標Y, 幅, 高さ
    ###############################
    def getBoxSlide(self, orgW, orgH) :
        margin = self.resizeBtn(10, orgW, orgH)
        barHeiget = self.resizeBtn(80, orgW, orgH)
        w = int(orgW - margin*2)
        h = margin
        sx = margin
        sy = int(orgH - barHeiget)
        return sx, sy, w, h

    ###############################
    # マウス操作
    #   event = マウスイベント
    #   x     = 座標位置X
    #   y     = 座標位置Y
    #   flags = イベントフラグ
    #   param = ユーザパラメータ
    ###############################
    def mouseClick(self, event, x, y, flags, param) :

        if (x >= 0 and y >= 0) :
            # ビデオパラメータ獲得
            fps, totalframes, orgW, orgH = param

            if event == cv2.EVENT_LBUTTONDOWN :
                # 切り出しボタン
                if self.inBox(x, y, self.getBoxWriteButton(orgW, orgH)) :
                    if self.writeFlag :
                        self.writeFlag = False
                    else :
                        self.writeFlag = True
                        self.writeCounter = 0
                # 読み飛ばしボタン
                elif self.inBox(x, y, self.getBoxFastButton(orgW, orgH))  :
                    self.frameNo += fps
                    self.writeCounter = 0
                # 巻き戻しボタン
                elif self.inBox(x, y, self.getBoxBackButton(orgW, orgH))  :
                    self.frameNo -= fps
                    if self.frameNo < 0 :
                        self.frameNo = 0
                    self.writeCounter = 0
                # スライド
                elif self.inBox(x, y, self.getBoxSlide(orgW, orgH)) :
                    sx, sy, w, h = self.getBoxSlide(orgW, orgH)
                    self.frameNo = int(((x - sx) / w) * totalframes)
                    self.writeCounter = 0
                # 開始、停止
                elif self.inBox(x, y, self.getBoxStartStop(orgW, orgH)) :
                    if self.moveFlag  :
                        # 開始中なら停止
                        self.moveFlag = False
                    else :
                        # 停止中なら開始
                        self.moveFlag = True
                        # 書き込み回数は初期化
                        self.writeCounter = 0

    
