##################################################
# 動画から画像を切り出す
#
# Shin.Onda 2023/6//3 all rights reserved.
##################################################
import tkinter as tk
import tkinter.messagebox as messagebox
import videotopicctrl
import os

# 画像のデフォルト初期値
defaultDir = os.path.abspath(os.path.dirname("./"))

# メインウインドウのボタン処理オブジェクト
root = tk.Tk()

###########################
# 入力値の検証
###########################
###########################
# 数値入力チェック
#   P = 入力値
# 戻り値 True 入力可、False 入力不可
###########################
def validateDegit(P) :
    ret = True

    if len(P) > 0 :
        if P.isdigit() :
            ret = True
        else :
            ret = False

    return ret

###########################
# 読み飛ばしフレーム数獲得
# 戻り値 読み飛ばしフレーム数
###########################
def getSkipCount() :
    t = txtSkipCount.get()
    if t is None :
        skipcount = 0;
    elif len(t) > 0 :
        skipcount = int(t)
    else :
        skipcount = 0;

    return skipcount

###########################
# サイズ調整がチェックされた場合
###########################
def cmdAdjustSize() :
    if blnAdjustSize.get() :
        txtAdjustSize['state'] = tk.NORMAL
    else :
        txtAdjustSize['state'] = tk.DISABLED

###########################
# 拡大縮小がチェックされた場合
###########################
def cmdMag() :
    if blnMag.get() :
        txtMax['state'] = tk.NORMAL
        txtMin['state'] = tk.NORMAL
    else :
        txtMax['state'] = tk.DISABLED
        txtMin['state'] = tk.DISABLED

###########################
# 画像切り出し処理
###########################
###########################
# 開始処理
#   btnFileRead = ファイル読み込みボタン
#   btnEnd      = 終了ボタン
###########################
def videoToPicStart(btnFileRead, btnEnd) :
    btnFileRead['state'] = tk.DISABLED
    btnEnd['state'] = tk.DISABLED

###########################
# 終了処理
#   btnFileRead = ファイル読み込みボタン
#   btnEnd      = 終了ボタン
###########################
def videoToPicEnd(btnFileRead, btnEnd) :
    btnFileRead['state'] = tk.NORMAL
    btnEnd['state'] = tk.NORMAL

###########################
# 動画読み込み・表示・ファイル書き込み
###########################
def videoToPic() :
    global defaultDir
    videoToPicCtrl = videotopicctrl.VideoToPicCtrl()
    
    # パラメータ設定
    videoToPicCtrl.setDefaultDir(defaultDir)
    videoToPicCtrl.setSkipCount(getSkipCount())
    videoToPicCtrl.setCutSquareOptin(blnCutSquareOption.get())
    videoToPicCtrl.setSquareOptin(blnSquareOption.get())
    videoToPicCtrl.setAdjustOption(blnAdjustSize.get())

    if blnAdjustSize.get() :
        if txtAdjustSize is None or len(txtAdjustSize.get()) < 1 :
            tk.Tk().withdraw()
            ret = messagebox.showinfo('確認', 'サイズを入力してください。')
            return
        if int(txtAdjustSize.get()) < 16 or int(txtAdjustSize.get()) > 4096 :
            tk.Tk().withdraw()
            ret = messagebox.showinfo('確認', 'サイズは16以上4096以下にしてください。')
            return
        videoToPicCtrl.setAdjustSize(int(txtAdjustSize.get()))

    videoToPicCtrl.setZoomOption(blnMag.get())
    if blnMag.get() :
        if (txtMin is None or txtMax is None) or (len(txtMin.get()) < 1 or len(txtMax.get()) < 1) :
            tk.Tk().withdraw()
            ret = messagebox.showinfo('確認', '拡大縮小の値を入力してください。')
            return
        if int(txtMin.get()) < 10 or int(txtMax.get()) > 1000 :
            tk.Tk().withdraw()
            ret = messagebox.showinfo('確認', '拡大縮小は10%以上1000%以下にしてください。')
            return
        if int(txtMin.get()) >= int(txtMax.get()) :
            tk.Tk().withdraw()
            ret = messagebox.showinfo('確認', '拡大縮小は、最小＜最大にしてください。')
            return
        videoToPicCtrl.setZoomMin(int(txtMin.get()))
        videoToPicCtrl.setZoomMax(int(txtMax.get()))

    # 開始処理
    videoToPicStart(btnFileRead, btnEnd)

    # 動画制御呼び出し
    videoToPicCtrl.videoCtrl()
    
    #終了処理
    defaultDir = videoToPicCtrl.getDefaultDir()
    videoToPicEnd(btnFileRead, btnEnd)


###########################
# メイン処理
###########################

# メイン画面とボタン
root.geometry('240x300')
root.title('動画画像切出')

# 数値検証
valCmd = root.register(validateDegit)

# ファイル選択ボタン
btnFileRead = tk.Button(root, text='ファイル選択', command=videoToPic)
btnFileRead.place(x=20, y=10)

# 終了ボタン
btnEnd = tk.Button(root, text='終了', command=root.quit)
btnEnd.place(x=180, y=10)

# 読み飛ばしフレーム数
lblSkipCount = tk.Label(text='読み飛ばしフレーム数')
lblSkipCount.place(x=20, y=50)
txtSkipCount = tk.Entry(width=4, validatecommand=(valCmd, '%P'), validate='all',)
txtSkipCount.place(x=160, y=50)
txtSkipCount.insert(0, '5')

# オプション
# 正方形切り出し
blnCutSquareOption = tk.BooleanVar()
blnCutSquareOption.set(False)
chkCutSquareOption = tk.Checkbutton(root, variable=blnCutSquareOption, text='正方形に切り出し(左、中央、右)')
chkCutSquareOption.place(x=20, y= 90)

# 正方形に配置
blnSquareOption = tk.BooleanVar()
blnSquareOption.set(False)
chkSquareOption = tk.Checkbutton(root, variable=blnSquareOption, text='正方形内に配置')
chkSquareOption.place(x=20, y=120)

# サイズ調整
blnAdjustSize = tk.BooleanVar()
blnAdjustSize.set(False)
chkAdjustSize = tk.Checkbutton(root, variable=blnAdjustSize, command = cmdAdjustSize, text='サイズ調整')
chkAdjustSize.place(x=20, y=150)
lblAdjustSize = tk.Label(text='サイズ調整（横Pixel値）')
lblAdjustSize.place(x=20, y=180)
txtAdjustSize = tk.Entry(width=4, validatecommand=(valCmd, '%P'), validate='all',)
txtAdjustSize.place(x=160, y=180)
txtAdjustSize.insert(0, '512')
txtAdjustSize['state'] = tk.DISABLED


# 拡大縮小
blnMag = tk.BooleanVar()
blnMag.set(False)
chkMag = tk.Checkbutton(root, variable=blnMag, command = cmdMag, text='拡大縮小(%)')
chkMag.place(x=20, y=210)
txtMin = tk.Entry(width=4, validatecommand=(valCmd, '%P'), validate='all',)
txtMin.place(x=80, y=240)
txtMin.insert(0, '50')
lblMag = tk.Label(text='～')
lblMag.place(x=120, y=240)
txtMax = tk.Entry(width=4, validatecommand=(valCmd, '%P'), validate='all',)
txtMax.place(x=160, y=240)
txtMax.insert(0, '150')
txtMax['state'] = tk.DISABLED
txtMin['state'] = tk.DISABLED

root.mainloop()

print('END.')
